// Public parameter definition - generated automatically

// Must compile with VC 2012 / GCC 4.8

#pragma once

#include "Ephere/Core/Parameters/Parameters.h"
#include "Ephere/Ornatrix/ParameterComponents.h"

namespace Ephere { namespace Ornatrix {
struct HairParameter;
struct PolygonMeshParameter;
class Ramp;
} }

namespace Ephere { namespace Ornatrix {

struct HairFromMeshStripsParameters
{
	enum { Version = 1 };

	enum { HighestId = 26, FirstUnusedId = 27 };

	static char const* GetName() { return "HairFromMeshStripsParameters"; }

	enum class DistributionMethodType
	{
		PerVertex,
		Uniform,
		Random,
	};

	struct DistributionMethod : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 1, DistributionMethodType>
	{
		static char const* Name() { return "DistributionMethod"; }

		static DistributionMethodType DefaultValue() { return DistributionMethodType::Random; }

		static char const* Description() { return ""; }
	};

	struct HairsPerStrip : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 2, int, Parameters::Direction::In, Parameters::Purpose::Undefined, false, true>
	{
		static char const* Name() { return "HairsPerStrip"; }

		static int DefaultValue() { return 100; }

		static char const* Description() { return ""; }
	};

	struct PointsPerStrand : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 3, int, Parameters::Direction::In, Parameters::Purpose::Undefined, false, true>
	{
		static char const* Name() { return "PointsPerStrand"; }

		static int DefaultValue() { return 5; }

		static char const* Description() { return ""; }
	};

	struct InheritTextureCoordinatesFromStrips : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 4, bool>
	{
		static char const* Name() { return "InheritTextureCoordinatesFromStrips"; }

		static bool DefaultValue() { return true; }

		static char const* Description() { return ""; }
	};

	struct Volume : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 5, float, Parameters::Direction::In, Parameters::Purpose::Undefined, false, true>
	{
		static char const* Name() { return "Volume"; }

		static float DefaultValue() { return 0; }

		static char const* Description() { return ""; }
	};

	struct VolumeNormalBias : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 6, float, Parameters::Direction::In, Parameters::Purpose::Undefined, false, true>
	{
		static char const* Name() { return "VolumeNormalBias"; }

		static float DefaultValue() { return 0; }

		static char const* Description() { return ""; }
	};

	struct RandomSeed : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 7, int, Parameters::Direction::In, Parameters::Purpose::RandomSeed, false, true>
	{
		static char const* Name() { return "RandomSeed"; }

		static int DefaultValue() { return 1; }

		static char const* Description() { return ""; }
	};

	// When enabled, a per-strand channel containing each hair's source mesh strip id will be created
	struct AddStripIdChannel : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 8, bool>
	{
		static char const* Name() { return "AddStripIdChannel"; }

		static bool DefaultValue() { return false; }

		static char const* Description() { return "When enabled, a per-strand channel containing each hair's source mesh strip id will be created"; }
	};

	struct IsUsingMaterialIdsAsStrandGroups : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 9, bool>
	{
		static char const* Name() { return "IsUsingMaterialIdsAsStrandGroups"; }

		static bool DefaultValue() { return false; }

		static char const* Description() { return ""; }
	};

	struct HairsPerStripFraction : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 10, float, Parameters::Direction::In, Parameters::Purpose::Fraction, false, true>
	{
		static char const* Name() { return "HairsPerStripFraction"; }

		static float DefaultValue() { return 1.0f; }

		static char const* Description() { return ""; }
	};

	struct GlobalVolumeDiagram : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 11, Ephere::Ornatrix::Ramp, Parameters::Direction::In, Parameters::Purpose::Undefined, false, true>
	{
		static char const* Name() { return "GlobalVolumeDiagram"; }

		static char const* DefaultValue() { return "0 0 0.5 0.5 1 1"; }

		static char const* Description() { return ""; }
	};

	struct OutputHair : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 25, Ephere::Ornatrix::HairParameter, Parameters::Direction::Out, Parameters::Purpose::Undefined, true>
	{
		static char const* Name() { return "OutputHair"; }

		static char const* Description() { return ""; }
	};

	struct objectToWorldTransform : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 26, Ephere::Ornatrix::Xform3, Parameters::Direction::In, Parameters::Purpose::Undefined, false, true>
	{
		static char const* Name() { return "objectToWorldTransform"; }

		static char const* Description() { return ""; }
	};

	struct StripMeshes
	{
		struct Mesh : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 23, Ephere::Ornatrix::PolygonMeshParameter[]>
		{
			static char const* Name() { return "StripMeshes.Mesh"; }

			static char const* Description() { return ""; }
		};

		struct UniqueId : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 24, int[]>
		{
			static char const* Name() { return "StripMeshes.UniqueId"; }

			static char const* Description() { return ""; }
		};
	};

	struct RootBoundaryIndices : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 12, int[]>
	{
		static char const* Name() { return "RootBoundaryIndices"; }

		static char const* Description() { return ""; }
	};

	// Strip group overrides
	struct StripsGroup
	{
		struct StripId : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 13, std::int64_t[]>
		{
			static char const* Name() { return "StripsGroup.StripId"; }

			static char const* Description() { return ""; }
		};

		struct GroupIndex : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 14, int[]>
		{
			static char const* Name() { return "StripsGroup.GroupIndex"; }

			static char const* Description() { return ""; }
		};
	};

	// Strip volume overrides
	struct StripsVolume
	{
		struct StripId : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 15, std::int64_t[]>
		{
			static char const* Name() { return "StripsVolume.StripId"; }

			static char const* Description() { return ""; }
		};

		struct Volume : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 16, float[]>
		{
			static char const* Name() { return "StripsVolume.Volume"; }

			static char const* Description() { return ""; }
		};
	};

	// Strip volume diagram overrides
	struct StripsVolumeDiagram
	{
		struct StripId : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 17, std::int64_t[]>
		{
			static char const* Name() { return "StripsVolumeDiagram.StripId"; }

			static char const* Description() { return ""; }
		};

		struct Diagram : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 18, Ephere::Ornatrix::Ramp[]>
		{
			static char const* Name() { return "StripsVolumeDiagram.Diagram"; }

			static char const* Description() { return ""; }
		};
	};

	// Strip hair count overrides
	struct StripsHairCount
	{
		struct StripId : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 19, std::int64_t[]>
		{
			static char const* Name() { return "StripsHairCount.StripId"; }

			static char const* Description() { return ""; }
		};

		struct HairCount : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 20, int[]>
		{
			static char const* Name() { return "StripsHairCount.HairCount"; }

			static char const* Description() { return ""; }
		};
	};

	// Strip distribution method overrides
	struct StripsDistributionMethod
	{
		struct StripId : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 21, std::int64_t[]>
		{
			static char const* Name() { return "StripsDistributionMethod.StripId"; }

			static char const* Description() { return ""; }
		};

		struct DistributionMethod : Parameters::ParameterDescriptor<HairFromMeshStripsParameters, 22, DistributionMethodType[]>
		{
			static char const* Name() { return "StripsDistributionMethod.DistributionMethod"; }

			static char const* Description() { return ""; }
		};
	};

	struct Descriptor;
	struct Container;
};

} }

namespace Ephere
{
template <> struct GetEnumInfo<Ephere::Ornatrix::HairFromMeshStripsParameters::DistributionMethodType>
{
	enum { Count = 3 };

	static char const* GetValueName( Ephere::Ornatrix::HairFromMeshStripsParameters::DistributionMethodType value )
	{
		using namespace Ephere::Ornatrix;
		switch( value )
		{
			case HairFromMeshStripsParameters::DistributionMethodType::PerVertex: return "PerVertex";
			case HairFromMeshStripsParameters::DistributionMethodType::Uniform: return "Uniform";
			case HairFromMeshStripsParameters::DistributionMethodType::Random: return "Random";
			default: return "";
		}
	}
};
}
