#include "Utilities.h"
#include "Ephere/Geometry/Native/MatrixUtilities.h"
#include "Ephere/Geometry/Native/PolygonMeshUtilities.h"
#include "Ephere/Geometry/Native/SharedPolygonMesh.h"
#include "Ephere/Geometry/Native/Test/Utilities.h"
#include "Ephere/Ornatrix/Operators/HairFromGuidesGenerator.h"
#include "Ephere/Ornatrix/Operators/MeshFromHairGenerator.h"
#include "Ephere/Ornatrix/Operators/SmoothSurface.h"
#include "Ephere/Ornatrix/Private/CommonHair.h"

namespace Ephere::Ornatrix
{
using namespace std;
using namespace Geometry;

TEST_CASE( "SmoothSurfacePreserveStrandRotations" )
{
	SharedHair hair;

	hair.SetSurfaceTangentComputeMethod( SurfaceTangentComputeMethod::FirstEdge );
	hair.SetUseSurfaceDependency2( true );

	auto const distributionMesh = make_shared<SharedPolygonMesh>();

	PolygonMeshUtilitiesT( *distributionMesh ).GeneratePlane( 1 );

	GenerateHairs( hair, array{ SurfacePosition { 0, { 0.5f, 0.5f } } }, 1, &*distributionMesh );
	MoveTips( hair, Vector3::XAxis() );

	hair.SetDistributionMesh( distributionMesh );

	hair.ValidateStrandToObjectTransforms( &*distributionMesh );

	HairFromGuidesGenerator hairFromGuides;
	hairFromGuides.rootParameters.distributionMesh = distributionMesh;
	hairFromGuides.rootParameters.generationMethod = RootGeneratorParameters::GenerationMethodType::FaceCenter;
	hairFromGuides.parameters.generatedStrandFraction = 1_r;
	hairFromGuides.parameters.inputGuides = &hair;

	auto const outputHair = make_shared<SharedHair>();
	outputHair->SetUsesGuideDependency( true );
	outputHair->SetUseSurfaceDependency2( true );
	outputHair->SetDistributionMesh( distributionMesh );
	outputHair->SetUseStrandToObjectTransforms( true );
	hairFromGuides.parameters.outputHair = outputHair;

	SharedHair sharedCacheHair;
	hairFromGuides.SetCacheHair( &sharedCacheHair );

	// Make sure we're using newer algorithm
	hairFromGuides.SetChangedGuides( true, nullptr, 0, 0, 0 );

	REQUIRE( hairFromGuides.Evaluate() );

	outputHair->ValidateStrandToObjectTransforms( &*distributionMesh );

	SmoothSurface op;
	op.parameters.subdivisionLevel.render = 1;
	op.Compute( *outputHair, true );

	outputHair->ValidateStrandToObjectTransforms( &*distributionMesh );

	REQUIRE_THAT(
		hair.GetVertices( IHair::CoordinateSpace::Object ),
		IsVectorNearEqual( outputHair->GetVertices( IHair::CoordinateSpace::Object ), 0.0001_r ) );
}

}
