struct Test_GroomModifier
(
	function TestInitialize =
	(
		OxResetClassParameters()
	),
	
	function TestCleanup =
	(
		delete $objects
		units.SystemType = #Centimeters
	),
	
	function TestBoxModifiers2 distributionObject firstModifier testUnbox:false =
	(
		modPanel.setCurrentObject firstModifier
		
		local verticesBeforeBoxing = OxGetVertices distributionObject true
		
		OxBoxModifiers()
		
		local verticesAfterBoxing = OxGetVertices distributionObject true
		
		EAssert.ArePointsAllNearEqual verticesBeforeBoxing verticesAfterBoxing
		
		local objectName = distributionObject.Name
		
		-- Save and reload the scene
		holdMaxFile()
		fetchMaxFile quiet:true
		
		distributionObject = getNodeByName objectName
		
		local verticesAfterReloading = OxGetVertices distributionObject true
		EAssert.ArePointsAllNearEqual verticesAfterBoxing verticesAfterReloading
		
		if testUnbox then
		(
			groom_modifier = distributionObject.modifiers[1]
			select distributionObject
			modPanel.setCurrentObject groom_modifier
			OxUnboxModifier()
			
			local verticesAfterUnboxing = OxGetVertices distributionObject true
			EAssert.ArePointsAllNearEqual verticesAfterBoxing verticesAfterUnboxing
		)
	),
	
	function TestBoxModifiers modifiers =
	(
		local distributionObject = Plane lengthsegs:1 widthsegs:1 mapcoords:on isSelected:on

		max modify mode

		-- Add gravity modifier to guides
		local guidesFromSurfaceModifier = Ox_Guides_from_Surface method:6 numSegs:2
		modPanel.addModToSelection guidesFromSurfaceModifier
		
		for addedModifier in modifiers do modPanel.addModToSelection addedModifier
		
		TestBoxModifiers2 distributionObject modifiers[1]
	),
	
	-- Box nodes command will generate a GroomModifier containing boxed modifiers
	function Test_BoxModifier =
	(
		TestBoxModifiers #( Ox_Strand_Gravity() )
	),
	
	function Test_BoxHairFromGuidesModifier =
	(
		TestBoxModifiers #( (Ox_Hair_from_Guides method:6 viewCountPercentage:100), Ox_Change_Width() )
		
		-- Reset the scene to check for a memory deallocation bug
		resetMaxFile #noPrompt
	),
	
	function Test_BoxClumpModifier =
	(
		local distributionObject = Plane lengthsegs:1 widthsegs:1 mapcoords:on isSelected:on

		max modify mode

		-- Add gravity modifier to guides
		modPanel.addModToSelection (Ox_Guides_from_Surface method:6 numSegs:2)
		modPanel.addModToSelection (Ox_Hair_from_Guides method:7 denseCount:50 viewCountPercentage:100)
		
		clump = Ox_Clump dynamicClumpingEnabled:on dynamicClumpingClumpCount:20 clumpCount:20 flyawayFraction:0
		modPanel.addModToSelection clump
		OxGetVertices distributionObject true
		clump.dynamicClumpingEnabled = off
		OxGetVertices distributionObject true
		
		TestBoxModifiers2 distributionObject clump
	),
	
	function Test_UnboxSurfaceCombModifier =
	(
		local distributionObject = Plane lengthsegs:1 widthsegs:1 mapcoords:on isSelected:on

		max modify mode

		-- Add gravity modifier to guides
		modPanel.addModToSelection (Ox_Guides_from_Surface method:6 numSegs:2)
		
		local surfaceCombInstance = Ox_Surface_Comb()
		modPanel.addModToSelection surfaceCombInstance
		surfaceCombInstance.CreateSinkAtCoordinate 0 [20,0,0] 0 [1,1]
		
		TestBoxModifiers2 distributionObject surfaceCombInstance testUnbox:true
	)
)

EUnit.AddTestClass Test_GroomModifier
ok
