#----------------
import os
import sys
import time

import c4d
sys.path.insert( 0, os.path.dirname( __file__ ) )
import Ornatrix.Ornatrix as Ox
import TestUtilities
#import importlib
#importlib.reload(TestUtilities)
sys.path.pop( 0 )
#----------------
from unittest import TestCase, skip

#
# TestCase
class Test_CollapseModifiers( TestCase ):
	doc = None
	hair = None
	gfm = None
	length = None
	eg = None
	hfg = None
	frizz = None
	cw = None

	#@classmethod
	#def setUpClass( self ):
	def setUp( self ):
		self.doc = c4d.documents.GetActiveDocument()
		# Delete all
		Ox.DeleteAll( self.doc )
		# Create Plane
		plane = c4d.BaseObject( c4d.Oplane )
		self.doc.InsertObject( plane )
		# Create Ornatrix Hair stack
		self.hair = c4d.BaseObject( Ox.res.ID_OX_HAIR_OBJECT )
		self.gfm = c4d.BaseObject( Ox.res.ID_OX_GUIDES_FROM_MESH_MOD )
		self.gfm[Ox.res.INPUT_MESH] = plane
		self.gfm[c4d.gfm_GuideCountAttribute] = 300
		self.length = c4d.BaseObject( Ox.res.ID_OX_LENGTH_MOD )
		self.length[Ox.res.len_ValueAttribute] = 2
		self.eg = c4d.BaseObject( Ox.res.ID_OX_EDIT_GUIDES_MOD )
		self.hfg = c4d.BaseObject( Ox.res.ID_OX_HAIR_FROM_GUIDES_MOD )
		self.hfg[c4d.hfg_RenderCountAttribute] = 1000
		self.hfg[c4d.hfg_ViewportCountFractionAttribute] = 1
		self.frizz = c4d.BaseObject( Ox.res.ID_OX_FRIZZ_MOD )
		self.cw = c4d.BaseObject( Ox.res.ID_OX_CHANGE_WIDTH_MOD )
		# Insert
		self.doc.InsertObject( self.hair )
		self.doc.InsertObject( self.cw, self.hair )
		self.doc.InsertObject( self.frizz, self.hair )
		self.doc.InsertObject( self.hfg, self.hair )
		self.doc.InsertObject( self.eg, self.hair )
		self.doc.InsertObject( self.length, self.hair )
		self.doc.InsertObject( self.gfm, self.hair )
		# Build
		Ox.ComputeStack( self.doc )

	def CollapseTo( self, selectedModifier ):
		self.doc.SetActiveObject( selectedModifier, c4d.SELECTION_NEW )
		TestUtilities.SendDescriptionCommand( selectedModifier, Ox.res.bake_Colapse_Button )
		Ox.ComputeStack( self.doc )
		# Cant find a way to properly end baking from Python, modifiers are still in the stack
		bakedHairOrGuides = selectedModifier.GetNext()
		return bakedHairOrGuides

	def test_GuidesOnly( self ):
		hairCountBefore = Ox.GetStrandCount( self.hair )
		guideCountBefore = Ox.GetStrandCount( self.eg )
		verticesBefore = TestUtilities.GetVerticesInObjectCoordinatesSorted( self.cw )
		# Collapse modifiers
		bakedGuides = self.CollapseTo( self.eg )
		# We cannot force end baking, so we could have collapsed the modifiers or have
		# an additional one.
		numberOfChildren = len( self.hair.GetChildren() )
		self.assertTrue( numberOfChildren == 7 or numberOfChildren == 4 )
		# Test baked hair and guide counts, should not be not affected by display fraction
		hairCountAfter = Ox.GetStrandCount( self.hair )
		guideCountAfter = Ox.GetStrandCount( bakedGuides )
		self.assertAlmostEqual( hairCountBefore, hairCountAfter, delta=3 )
		self.assertAlmostEqual( guideCountBefore, guideCountAfter, delta=1 )
		# Test BakedHair output, must reflect display fraction
		verticesAfter = TestUtilities.GetVerticesInObjectCoordinatesSorted( self.cw )
		TestUtilities.CheckPointsAllNearEqual( self, verticesBefore, verticesAfter )

	def test_CollapseHair( self ):
		hairCountBefore = Ox.GetStrandCount( self.hair )
		verticesBefore = TestUtilities.GetVerticesInObjectCoordinatesSorted( self.cw )
		# Bake modifiers
		self.CollapseTo( self.cw )
		# We cannot force end baking, so we could have collapsed the modifiers or have
		# an additional one.
		numberOfChildren = len( self.hair.GetChildren() )
		self.assertTrue( numberOfChildren == 7 or numberOfChildren == 1 )
		# Test baked hair count, should not be not affected by display fraction
		hairCountAfter = Ox.GetStrandCount( self.hair )
		self.assertAlmostEqual( hairCountBefore, hairCountAfter, delta=3 )
		# Test BakedHair output, must reflect display fraction
		verticesAfter = TestUtilities.GetVerticesInObjectCoordinatesSorted( self.cw )
		TestUtilities.CheckPointsAllNearEqual( self, verticesBefore, verticesAfter )

	def test_CollapseZeroGuides( self ):
		self.gfm[Ox.res.gfm_GuideCountAttribute] = 0
		Ox.ComputeStack( self.doc )
		self.CollapseTo( self.eg )
		self.assertEqual( len( self.hair.GetChildren() ), 6 )
		hairCountAfter = Ox.GetStrandCount( self.hair )
		self.assertEqual( hairCountAfter, 0 )

	def test_CollapseZeroHairs( self ):
		self.gfm[Ox.res.gfm_GuideCountAttribute] = 0
		Ox.ComputeStack( self.doc )
		self.CollapseTo( self.cw )
		self.assertEqual( len( self.hair.GetChildren() ), 6 )
		hairCountAfter = Ox.GetStrandCount( self.hair )
		self.assertEqual( hairCountAfter, 0 )


#----------------------------------
# main
testObject = None
if __name__ == '__main__':
	Ox.runTest( doc, Test_BakeModifiers )