#----------------
import os
import sys
import c4d
sys.path.insert( 0, os.path.dirname( __file__ ) )
import Ornatrix.Ornatrix as Ox
import TestUtilities
sys.path.pop( 0 )
#----------------
from unittest import TestCase

#
# TestCase
class Test_CurlModifier( TestCase ):

	def setUp( self ):
		self.doc = c4d.documents.GetActiveDocument()
		Ox.DeleteAll( self.doc )
		# Create base object
		plane = c4d.BaseObject( c4d.Oplane )
		self.doc.InsertObject( plane )
		plane[c4d.PRIM_PLANE_WIDTH] = 100
		plane[c4d.PRIM_PLANE_HEIGHT] = 100
		plane[c4d.PRIM_PLANE_SUBW] = 3
		plane[c4d.PRIM_PLANE_SUBH] = 3
		res = c4d.utils.SendModelingCommand(
				command = c4d.MCOMMAND_MAKEEDITABLE,
				list = [plane],
				mode = c4d.MODELINGCOMMANDMODE_ALL,
				doc = self.doc )

		if res is False or res is True:
			raise TypeError( "Test_CurlModifier: Error when making plane editable." )
		elif isinstance(res, list):
			self.plane = res[0]
			self.doc.InsertObject( self.plane )
		c4d.EventAdd()

		self.hair = TestUtilities.AddGuidesToMesh( self.plane, guideCount=20, pointsPerStrandCount=10 )
		self.gfm = self.hair.GetDown()
		# Add clump
		self.curl = c4d.BaseObject( Ox.res.ID_OX_CURL_MOD )
		self.curl.InsertUnderLast( self.hair )

		# Build
		Ox.ComputeStack( self.doc )
		self.verticesBeforeCurl = TestUtilities.GetVerticesInObjectCoordinatesByStrandId( self.gfm )
		self.verticesAfterCurl = TestUtilities.GetVerticesInObjectCoordinatesByStrandId( self.curl )

	def tearDown( self ):
		#Ox.DeleteAll( self.doc )
		self.doc = None

	# Test if Clump modifier changed original guides
	def test_StrandsChanged( self ):
		TestUtilities.CheckPointsNotAllNearEqualByStrandId( self, self.verticesBeforeCurl, self.verticesAfterCurl )

	def test_VertexColorChannels( self ):
		TestUtilities.CreateVertexColors( self.doc, self.plane, 8 )
		verticesBeforeCurl = TestUtilities.GetVerticesInObjectCoordinates( self.gfm )
		verticesBeforeCurlInBlackArea = [v for v in verticesBeforeCurl if v.z < 0]
		curledVertices = TestUtilities.GetVerticesInObjectCoordinates( self.curl )
		curledVerticesInBlackArea = [v for v in curledVertices if v.z < 0]

		# Test magnitude channel
		self.curl[Ox.res.crl_MagnitudeChannelAttribute] = TestUtilities.Ox_FirstVertexColorChannelIndex
		Ox.ComputeStack( self.doc )
		newVertices = TestUtilities.GetVerticesInObjectCoordinates( self.curl )
		newVerticesInBlackArea = [v for v in newVertices if v.z < 0]

		TestUtilities.CheckPointsNotAllNearEqual( self, curledVerticesInBlackArea, newVerticesInBlackArea )
		TestUtilities.CheckPointsAllNearEqual( self, verticesBeforeCurlInBlackArea, newVerticesInBlackArea )
		self.curl[Ox.res.crl_MagnitudeChannelAttribute] = 0

		# Test phase channel
		self.curl[Ox.res.crl_PhaseAttribute] = 0
		Ox.ComputeStack( self.doc )
		verticesWithoutPhase = TestUtilities.GetVerticesInObjectCoordinates( self.curl )
		verticesWithoutPhaseInBlackArea = [v for v in verticesWithoutPhase if v.z < 0]

		self.curl[Ox.res.crl_PhaseChannelAttribute] = TestUtilities.Ox_FirstVertexColorChannelIndex
		self.curl[Ox.res.crl_PhaseAttribute] = 2.0
		Ox.ComputeStack( self.doc )
		newVertices = TestUtilities.GetVerticesInObjectCoordinates( self.curl )
		newVerticesInBlackArea = [v for v in newVertices if v.z < 0]

		TestUtilities.CheckPointsNotAllNearEqual( self, curledVerticesInBlackArea, newVerticesInBlackArea )
		TestUtilities.CheckPointsAllNearEqual( self, verticesWithoutPhaseInBlackArea, newVerticesInBlackArea )
		self.curl[Ox.res.crl_PhaseChannelAttribute] = 0

		# Test stretch channel
		self.curl[Ox.res.crl_StretchAttribute] = 0.0
		Ox.ComputeStack( self.doc )
		verticesWithoutStretch = TestUtilities.GetVerticesInObjectCoordinates( self.curl )
		verticesWithoutStretchInBlackArea = [v for v in verticesWithoutStretch if v.z < 0]

		self.curl[Ox.res.crl_StretchChannelAttribute] = TestUtilities.Ox_FirstVertexColorChannelIndex
		self.curl[Ox.res.crl_StretchAttribute] = 1.0
		Ox.ComputeStack( self.doc )
		newVertices = TestUtilities.GetVerticesInObjectCoordinates( self.curl )
		newVerticesInBlackArea = [v for v in newVertices if v.z < 0]

		TestUtilities.CheckPointsAllNearEqual( self, verticesWithoutStretchInBlackArea, newVerticesInBlackArea )
		self.curl[Ox.res.crl_StretchChannelAttribute] = 0



#----------------------------------
# main
testObject = None
if __name__ == '__main__':
	Ox.runTest( doc, Test_CurlModifier )