#----------------
import os
import sys
import c4d
sys.path.insert( 0, os.path.dirname( __file__ ) )
import Ornatrix.Ornatrix as Ox
import TestUtilities
sys.path.pop( 0 )
#----------------
from unittest import TestCase

#
# TestCase
class Test_HFG_VertexColorChannel( TestCase ):
	doc = None
	hair = None
	gfm = None
	hfg = None

	def setUp( self ):
		self.doc = c4d.documents.GetActiveDocument()
		# Delete all
		Ox.DeleteAll( self.doc )
		# Create plane
		plane = c4d.BaseObject( c4d.Oplane )
		self.planeWidth = 100
		self.numSegments = 3
		plane[c4d.PRIM_PLANE_WIDTH] = self.planeWidth
		plane[c4d.PRIM_PLANE_HEIGHT] = self.planeWidth
		plane[c4d.PRIM_PLANE_SUBW] = self.numSegments
		plane[c4d.PRIM_PLANE_SUBH] = self.numSegments
		self.doc.InsertObject( plane )
		res = c4d.utils.SendModelingCommand(
				command = c4d.MCOMMAND_MAKEEDITABLE,
				list = [plane],
				mode = c4d.MODELINGCOMMANDMODE_ALL,
				doc = self.doc )

		if res is False or res is True:
			raise TypeError( "Test_HFG_VertexColorChannel: Error when making plane editable." )
		elif isinstance(res, list):
			self.plane = res[0]
			self.doc.InsertObject( self.plane )
		c4d.EventAdd()

		# Create Ornatrix Hair object
		self.hair = c4d.BaseObject( Ox.res.ID_OX_HAIR_OBJECT )
		self.doc.InsertObject( self.hair )
		# Create GFM modifier inside Hair Object
		self.gfm = c4d.BaseObject( Ox.res.ID_OX_GUIDES_FROM_MESH_MOD )
		self.gfm[Ox.res.gfm_RootGenerationMethodAttribute] = 6 # Face center roots
		self.gfm[Ox.res.gfm_GuidePointCountAttribute] = 5 # 5 points per strand
		self.gfm[Ox.res.INPUT_MESH] = self.plane
		self.hfg = c4d.BaseObject( Ox.res.ID_OX_HAIR_FROM_GUIDES_MOD )
		self.doc.InsertObject( self.hfg, self.hair )
		self.doc.InsertObject( self.gfm, self.hair )
		# Build
		Ox.ComputeStack( self.doc )

	def test_VertexColorChannel( self ):
		segmentWidth = self.planeWidth / self.numSegments
		planeMinBound = -self.planeWidth / 2.0
		blackMaxBound = planeMinBound + segmentWidth

		# Color the first column of faces black
		TestUtilities.CreateVertexColors( self.doc, self.plane, 4 + 2 * ( self.numSegments - 1 ) )

		# Test there are no strands inside the black area
		self.hfg[Ox.res.hfg_DistributionChannelAttribute] = TestUtilities.Ox_FirstVertexColorChannelIndex
		Ox.ComputeStack( self.doc )
		newRootIdsInBlackArea = TestUtilities.GetAndFilterRootsByMaxBound( self.hfg, blackMaxBound, coordinateIndex=2 )

		self.assertEqual( len( newRootIdsInBlackArea ), 0 )

		self.hfg[Ox.res.hfg_RootGenerationMethodAttribute] = 4 # Vertex distribution
		self.hfg[Ox.res.hfg_DistributionChannelAttribute] = 0
		Ox.ComputeStack( self.doc )
		rootIdsInWhiteArea = TestUtilities.GetAndFilterRootsByMinBound( self.hfg, 0, coordinateIndex=2 )

		self.hfg[Ox.res.hfg_DistributionChannelAttribute] = TestUtilities.Ox_FirstVertexColorChannelIndex
		Ox.ComputeStack( self.doc )
		newRootIdsInWhiteArea = TestUtilities.GetAndFilterRootsByMinBound( self.hfg, 0, coordinateIndex=2 )
		self.assertEqual( newRootIdsInWhiteArea, rootIdsInWhiteArea )


#----------------------------------
# main
testObject = None
if __name__ == '__main__':
	Ox.runTest( doc, Test_HFG_VertexColorChannel )
