#----------------
import os
import sys
import math
import c4d
sys.path.insert( 0, os.path.dirname( __file__ ) )
import Ornatrix.Ornatrix as Ox
import TestUtilities
sys.path.pop( 0 )
#----------------
from unittest import TestCase

#
# TestCase
class Test_PropagationModifier( TestCase ):

	def setUp( self ):
		self.doc = c4d.documents.GetActiveDocument()
		Ox.DeleteAll( self.doc )
		# Create base object
		self.hair = TestUtilities.AddGuidesToNewPlane( planeSize=100, planeSegmentCount=3, guideCount=20, pointsPerStrandCount=5, rootGenerationMethod=6 )
		self.gfm = self.hair.GetDown()
		# Add propagator
		self.propagator = c4d.BaseObject( Ox.res.ID_OX_PROPAGATION_MOD )
		self.propagator[Ox.res.prop_RootCountAttribute] = 5
		self.propagator.InsertUnderLast( self.hair )
		# Build
		Ox.ComputeStack( self.doc )

	def tearDown( self ):
		self.doc = None

	def test_VertexColorChannels( self ):
		plane = self.gfm[Ox.res.gfm_InputMeshAttribute]
		numSegments = plane[c4d.PRIM_PLANE_SUBW]
		planeWidth = plane[c4d.PRIM_PLANE_WIDTH]
		segmentWidth = planeWidth / numSegments
		planeMinBound = -planeWidth / 2.0
		blackMaxBound = planeMinBound + segmentWidth

		# Make the plane modifiable
		res = c4d.utils.SendModelingCommand(
				command = c4d.MCOMMAND_MAKEEDITABLE,
				list = [plane],
				mode = c4d.MODELINGCOMMANDMODE_ALL,
				doc = self.doc )

		if res is False or res is True:
			raise TypeError( "Test_PropagationModifier: Error when making plane editable." )
		elif isinstance(res, list):
			plane = res[0]
			self.doc.InsertObject( plane )
		c4d.EventAdd()

		# Color the first column of faces black and get vertices in that area
		TestUtilities.CreateVertexColors( self.doc, plane, 4 + 2 * ( numSegments - 1 ) )
		rootIdsInBlackArea = TestUtilities.GetAndFilterRootsByMaxBound( self.gfm, blackMaxBound, 2 )
		rootIdsInWhiteAreaBeforeProp = TestUtilities.GetAndFilterRootsByMinBound( self.gfm, -blackMaxBound, 2 )
		rootIdsInWhiteArea = TestUtilities.GetAndFilterRootsByMinBound( self.propagator, -blackMaxBound, 2 )
		verticesBeforePropagatorInBlackArea = TestUtilities.GetVerticesAndFilterByIds( self.gfm, rootIdsInBlackArea )
		verticesAfterPropagatorInWhiteArea = TestUtilities.GetVerticesAndFilterByIds( self.propagator, rootIdsInWhiteArea )

		# Test probability channel
		self.propagator[Ox.res.prop_DistributionChannelAttribute] = TestUtilities.Ox_FirstVertexColorChannelIndex
		Ox.ComputeStack( self.doc )
		newRootsInBlackArea = TestUtilities.GetAndFilterRootsByMaxBound( self.propagator, blackMaxBound, 2 )
		newRootsInWhiteArea = TestUtilities.GetAndFilterRootsByMinBound( self.propagator, -blackMaxBound, 2 )
		newVerticesInBlackArea = TestUtilities.GetVerticesAndFilterByIds( self.propagator, newRootsInBlackArea )
		newVerticesInWhiteArea = TestUtilities.GetVerticesAndFilterByIds( self.propagator, newRootsInWhiteArea )
		self.assertEqual( len( rootIdsInBlackArea ), len( newRootsInBlackArea ) )
		self.assertNotEqual( len( rootIdsInWhiteAreaBeforeProp ), len( newRootsInWhiteArea ) )
		self.assertEqual( len( rootIdsInWhiteArea ), len( newRootsInWhiteArea ) )
		TestUtilities.CheckPointsAllNearEqual( self, verticesBeforePropagatorInBlackArea, newVerticesInBlackArea )
		TestUtilities.CheckPointsAllNearEqual( self, verticesAfterPropagatorInWhiteArea, newVerticesInWhiteArea )
		self.propagator[Ox.res.prop_DistributionChannelAttribute] = 0



#----------------------------------
# main
testObject = None
if __name__ == '__main__':
	Ox.runTest( doc, Test_PropagationModifier )
