#----------------
import os
import sys
import math
import c4d
sys.path.insert( 0, os.path.dirname( __file__ ) )
import Ornatrix.Ornatrix as Ox
import TestUtilities
sys.path.pop( 0 )
#----------------
from unittest import TestCase

#
# TestCase
class Test_WidthModifier( TestCase ):

	def setUp( self ):
		self.doc = c4d.documents.GetActiveDocument()
		Ox.DeleteAll( self.doc )
		# Create base object
		self.hair = TestUtilities.AddGuidesToNewPlane( planeSize=100, planeSegmentCount=3, guideCount=20, pointsPerStrandCount=5, rootGenerationMethod=6 )
		self.gfm = self.hair.GetDown()
		# Add hair
		self.hfg = c4d.BaseObject( Ox.res.ID_OX_HAIR_FROM_GUIDES_MOD)
		self.hfg[Ox.res.hfg_RootGenerationMethodAttribute] = Ox.res.hfg_RootGenerationMethodAttribute_GUIDE_AS_HAIR
		self.hfg.InsertUnderLast( self.hair )
		# Add width
		self.width = c4d.BaseObject( Ox.res.ID_OX_CHANGE_WIDTH_MOD)
		self.width[Ox.res.chw_WidthAttribute] = 1.5
		self.width.InsertUnderLast( self.hair )
		# Build
		Ox.ComputeStack( self.doc )

	def tearDown( self ):
		self.doc = None

	def test_VertexColorChannels( self ):
		plane = self.gfm[Ox.res.gfm_InputMeshAttribute]
		numSegments = plane[c4d.PRIM_PLANE_SUBW]
		planeWidth = plane[c4d.PRIM_PLANE_WIDTH]
		segmentWidth = planeWidth / numSegments
		planeMinBound = -planeWidth / 2.0
		blackMaxBound = planeMinBound + segmentWidth

		# Make the plane modifiable
		res = c4d.utils.SendModelingCommand(
				command = c4d.MCOMMAND_MAKEEDITABLE,
				list = [plane],
				mode = c4d.MODELINGCOMMANDMODE_ALL,
				doc = self.doc )

		if res is False or res is True:
			raise TypeError( "Test_WidthModifier: Error when making plane editable." )
		elif isinstance(res, list):
			plane = res[0]
			self.doc.InsertObject( plane )
		c4d.EventAdd()

		# Color the first column of faces black and get vertices in that area
		TestUtilities.CreateVertexColors( self.doc, plane, 4 + 2 * ( numSegments - 1 ) )
		rootIdsInBlackArea = TestUtilities.GetAndFilterRootsByMaxBound( self.width, blackMaxBound, 2 )
		rootIdsInWhiteArea = TestUtilities.GetAndFilterRootsByMinBound( self.width, -blackMaxBound, 2 )

		# Test width channel
		widthsBeforeChannel = TestUtilities.GetWidthsByStrandId( self.width )
		blackWidthsBeforeChannel = TestUtilities.FilterAndFlattenDictionaryOfLists( widthsBeforeChannel, rootIdsInBlackArea )
		whiteWidthsBeforeChannel = TestUtilities.FilterAndFlattenDictionaryOfLists( widthsBeforeChannel, rootIdsInWhiteArea )
		self.width[Ox.res.chw_WidthChannelAttribute] = TestUtilities.Ox_FirstVertexColorChannelIndex
		Ox.ComputeStack( self.doc )
		widthsAfterChannel = TestUtilities.GetWidthsByStrandId( self.width )
		blackWidthsAfterChannel = TestUtilities.FilterAndFlattenDictionaryOfLists( widthsAfterChannel, rootIdsInBlackArea )
		whiteWidthsAfterChannel = TestUtilities.FilterAndFlattenDictionaryOfLists( widthsAfterChannel, rootIdsInWhiteArea )
		self.assertEqual( len( blackWidthsBeforeChannel ), len( blackWidthsAfterChannel ) )
		self.assertEqual( len( whiteWidthsBeforeChannel ), len( whiteWidthsAfterChannel ) )
		blackWidthsChanged = False
		for i, width in enumerate( blackWidthsBeforeChannel ):
			epsilon = 0.01

			widthAfter = blackWidthsAfterChannel[i]
			error = abs( width - widthAfter )
			blackWidthsChanged = blackWidthsChanged or error > epsilon

			widthBefore = whiteWidthsBeforeChannel[i]
			widthAfter = whiteWidthsAfterChannel[i]
			error = abs( widthBefore - widthAfter )
			self.assertGreaterEqual( epsilon, error, msg = f"[{str( widthBefore )}] is not near equal [{str( widthAfter )}], error: {str( error )}" )

		self.assertTrue( blackWidthsChanged )
		self.width[Ox.res.chw_WidthChannelAttribute] = 0



#----------------------------------
# main
testObject = None
if __name__ == '__main__':
	Ox.runTest( doc, Test_WidthModifier )
